#include "IVR.h"
#include <libxml/tree.h>
#include <libxml/parser.h>


IVR::IVR(std::string xmlFile)
{
    mRootStep = 0;
    xmlDocPtr doc = xmlReadFile(xmlFile.c_str(),0,0);
    if (doc)
    {
        xmlNode *root = xmlDocGetRootElement(doc);
        mRootStep = getIVRStepFromXml(root->children);
        xmlFreeDoc(doc);
    }

    /*mRootStep = new IVRStep("Press 1, or enter time (hhmm)","","",None);
    mRootStep->addSubStep(new IVRStep("Using default. Thank you","1","",None));
    IVRStep *step2 = new IVRStep("enter extension","xxxx","time",None);
    step2->addSubStep(new IVRStep("Thank you","*","extension",Hangup));
    mRootStep->addSubStep(step2);*/

    mCurrentStep = mRootStep;

//    showIVR(mCurrentStep);


}

IVR::~IVR(){
    if (mRootStep) delete mRootStep;
    mRootStep = 0;
}


void IVR::showIVR(IVRStep *step)
{
    printf("Step: %s\r\n",step->toString().c_str());
    std::list<IVRStep*> list = step->getSteps();
    for (std::list<IVRStep*>::iterator it=list.begin();it!=list.end();it++){
        showIVR(*it);
    }
}


IVRStep* IVR::getIVRStepFromXml(xmlNode * node)
{
    if (!node) return 0;
    xmlNode *curnode;
    IVRStep* step = new IVRStep();  

    char *st = (char*)xmlGetProp(node->parent,(const xmlChar*)"keep");
    if (st) step->setKeep(st);
    st = (char*)xmlGetProp(node->parent,(const xmlChar*)"match");
    if (st) step->setPattern(st);
    

    curnode = node;
    while (curnode = curnode->next)
    {
        std::string nodename = (char*)curnode->name;
        if (nodename=="prompt")
        {
            step->setPrompt((char*)curnode->children->content);
        }
        if (nodename=="exit")
        {
            step->setAction(Hangup);
        }

        if (nodename=="step")
        {
            IVRStep *child = this->getIVRStepFromXml(curnode->children);
            if (child)
            {
                step->addSubStep(child);
            }
        }
    }

    return step;

}

std::string IVR::getUserData(std::string key)
{
    if (mUserData.find(key)!=mUserData.end())
    {
        return mUserData[key];
    }
    return "";
}

std::string IVR::getPrompt()
{
    if (mCurrentStep)
    {
        return mCurrentStep->getPrompt();
    }
}

IVRResult IVR::onDigits(std::string digits)
{
    if (!mCurrentStep) return Terminated;

    IVRStep* step = mCurrentStep->onDigits(digits);

    if (mCurrentStep != step)
    {
        if (step)
        {
            std::string key = mCurrentStep->getKeepName();
            if (key!="")
            {
                if (digits[digits.size()-1]=='#')
                {
                    digits.resize(digits.size()-1);
                }
                this->mUserData[key] = digits;
            }
        }

        mCurrentStep = step;
        if (mCurrentStep->getAction()==None){
            return SwitchStep;
        } else if (mCurrentStep->getAction()==Hangup){
            return HangupCall;
        } else if (mCurrentStep->getAction()==NoSubStep){
            return Terminated;
        }
    } else {
        return WaitingForMore;
    }
}

