#include "WakeupCallService.h"
#include "AppDialogSetEndCommand.h"
#include "config.h"
#include <sstream>

WakeupCallService::WakeupCallService(CallManager *callManager){
    this->mCallManager = callManager;
    this->mCallManager->addObserver(this);
    
    
//    this->addWakeupCall(new WakeupCall("710",100));
  //  this->addWakeupCall(new WakeupCall("704",20));
}

WakeupCallService::~WakeupCallService(){
}


void WakeupCallService::addWakeupCall(WakeupCall *wc)
{
    std::string id = wc->getID();
    //TODO: delete previous instance if it exists
    this->mWakeupCalls[id] = wc;  

    printf("Adding wakeupCall for %s at %i\r\n",wc->getExtension().c_str(),wc->getTime());

    wc->setActive();
}

void WakeupCallService::onRTPSessionSoundQueueEmpty(Call *call)
{
    //WARNING: This is called from the RTP worker Thread

    if (call->isIncomming())
    {

    } else {
        printf("Sound queue empty. Terminating call\r\n");
        this->mCallManager->release(call);
    }
}


void WakeupCallService::work()
{
    /*if (test){
        Call *call = this->mCallManager->call("704");
        if (call){
            call->addRTPObserver(this);
            test = false;
        }
    }*/


    time_t t;
    time(&t);
    for (std::map<std::string,WakeupCall*>::iterator it=mWakeupCalls.begin();it!=mWakeupCalls.end();it++)
    {
        WakeupCall *wc = it->second;
        if (wc->getTime()>t) break;
        if (wc->isActive())
        {
            Call *call = this->mCallManager->call(wc->getExtension());
            if (call){
                printf("Wakup Call for %s at %i\r\n",wc->getExtension().c_str(),wc->getTime());
                call->addRTPObserver(this);
                delete wc;
                mWakeupCalls.erase(it);
            }
        } else {
            delete wc;
            mWakeupCalls.erase(it);
        }
    }

}

void WakeupCallService::onIncommingCallAnswered(Call *call)
{
    printf("WakeupCallService::onIncommingCallAnswered\r\n");

    this->mIVRMap[call]=new IVR("ivr.xml");
    call->getRTPSession()->play(this->mIVRMap[call]->getPrompt().c_str());
}

void WakeupCallService::onIncommingCall(Call *call)
{
    call->addRTPObserver(this);
    this->mCallManager->answerIncomming(call);
}


void WakeupCallService::onCallTerminated(Call *call)
{

    printf("WakeupCallService::onCallTerminated\r\n");

    std::map<Call*,IVR*>::iterator it = this->mIVRMap.find(call);
    if (it != this->mIVRMap.end())
    {
        IVR* ivr = this->mIVRMap[call];
        this->mIVRMap.erase(it);
        delete ivr;
    }

}

void WakeupCallService::onCallAnswered(Call *call)
{
    // Wakeup call ansered. Play sound file.
    printf("WakeupCallService::onCallAnswered()\r\n");

    call->getRTPSession()->silence(1);
    std::stringstream ss;
    ss << SOUND_PATH << SOUND_WAKEUP;
    call->getRTPSession()->play(ss.str().c_str());

}

void WakeupCallService::onCallFailed(Call *call)
{
    printf("WakeupCallService::onCallFailed\r\n");
}

void WakeupCallService::onDTMF(Call *call,std::string digit)
{
    
    if (call->isIncomming())
    {
        call->getRTPSession()->clearQueue();
        std::string digits = call->getDigitQueue();

        std::map<Call*,IVR*>::iterator it = this->mIVRMap.find(call);
        if (it != this->mIVRMap.end())
        {
            IVR* ivr = this->mIVRMap[call];
//    printf("testing digits [%s]\r\n",digits.c_str());
            IVRResult result = ivr->onDigits(digits);
            if (result==Terminated)
            {
                call->clearDigitQueue();
                std::string choice = ivr->getUserData("choice");
                std::string time = ivr->getUserData("time");
                std::string extension = ivr->getUserData("extension");
                if (choice=="1")
                {
                    /*const char* c = time.c_str();
                    for (int i=0;i<digits.size();i++)
                    {
                        if (c[i]>=0x30 && c[i]<=0x39)
                        {
                            call->getRTPSession()->playNumber(c[i]-48);
                        }
                    }*/
                    //TODO: compute time
                    int h=((time[0]-48)*10)+(time[1]-48);
                    int m=((time[2]-48)*10)+(time[3]-48);
                    time_t t = this->buildTime(h,m);
                    
                    this->addWakeupCall(new WakeupCall(extension,t));
                    this->mCallManager->release(call);
                }
            } else if (result==HangupCall)
            {
                call->clearDigitQueue();
            } else if (result==SwitchStep)
            {
                printf("Switch Step: %s\r\n",ivr->getPrompt().c_str());
                call->clearDigitQueue();
                call->getRTPSession()->play(ivr->getPrompt().c_str());
            } else if (result==WaitingForMore)
            {
                //User needs to input more
            }
        }
    }
}


time_t WakeupCallService::buildTime(int h, int m)
{
    time_t t;
    struct tm *tinfo;
    time(&t);
    tinfo = localtime(&t);
    tinfo->tm_hour = h;
    tinfo->tm_min = m;
    tinfo->tm_sec=0;
    
    time_t t2 = mktime(tinfo);
    if (t2<t)
    {
        t2+=(24*60*60);
    }


    return t2;
}
